' Gap / Silence Between Songs script v3.0
' By Eyal, 2009.09.16 - Updated 2010.12.26
' By Onno Tabak 2010-04-21 with small part of Gap.vbs by Soren Werk
'
' This script adds an entry in Play menu that let you
' enable/disable a silence time between playing songs.
' Delay time is configurable through Options|Player|SBS.
' Requires MediaMonkey 3.1.2 or newer.
'
' Repeat and Shuffle is accounted for.
' Next song selected before gap
'
' Location: MediaMonkey\Scripts\Auto\SilenceBetweenSongs.vbs
'---------------------
Option Explicit

Const DEFAULTCustomFieldText = "[Gapless]"
'Set Silence Time in seconds (default = 3):
Dim SilenceTime : SilenceTime = 3
Dim SilenceEnabled : SilenceEnabled = False
Dim GapProgress
Dim GapTimer
Dim CurrentTrack
Dim ExcludeGapless : ExcludeGapless = False
Dim isGapless
Dim CustomFieldToUse : CustomFieldToUse = 3
Dim CustomFieldText : CustomFieldText = DEFAULTCustomFieldText

Dim AppTitle : AppTitle = "SilenceBetweenSongs"
Dim Version : Version = "3.0"
Dim MenuItem

'---------------------
Sub OnStartup
   InitButton
   SilenceBetweenSongs
   InitTimer
End Sub

'---------------------
Sub InitButton()
   SDB.IniFile.StringValue(AppTitle,"Version") = Version   '"v3.0"

    If Not SDB.IniFile.ValueExists(AppTitle,"Enabled") Then
       SDB.IniFile.BoolValue(AppTitle,"Enabled") = SilenceEnabled
    End If

    If Not SDB.IniFile.ValueExists(AppTitle,"SilenceTime") Then
       SDB.IniFile.StringValue(AppTitle,"SilenceTime") = SilenceTime
    End If

    If Not SDB.IniFile.ValueExists(AppTitle,"CrossfadeState") Then
       SDB.IniFile.BoolValue(AppTitle,"CrossfadeState") = SDB.Player.IsCrossfade
    End If

    If Not SDB.IniFile.ValueExists(AppTitle,"ExcludeGapless") Then
       SDB.IniFile.BoolValue(AppTitle,"ExcludeGapless") = ExcludeGapless
    End If

    If Not SDB.IniFile.ValueExists(AppTitle,"CustomFieldToUse") Then
       SDB.IniFile.StringValue(AppTitle,"CustomFieldToUse") = CustomFieldToUse
    End If

    If Not SDB.IniFile.ValueExists(AppTitle,"CustomFieldText") Then
       SDB.IniFile.StringValue(AppTitle,"CustomFieldText") = CustomFieldText
    End If

    SilenceTime = SDB.IniFile.StringValue(AppTitle,"SilenceTime")
    SilenceEnabled = SDB.IniFile.BoolValue(AppTitle,"Enabled")
    ExcludeGapless = SDB.IniFile.BoolValue(AppTitle,"ExcludeGapless")
    CustomFieldToUse = SDB.IniFile.StringValue(AppTitle,"CustomFieldToUse")
    CustomFieldText = SDB.IniFile.StringValue(AppTitle,"CustomFieldText")
        
    Set MenuItem = SDB.UI.AddMenuItem(SDB.UI.Menu_Play,4,2)
    MenuItem.Caption = "Silence between songs"
    Script.RegisterEvent MenuItem, "OnClick", "ToggleSilence"
    MenuItem.Visible = True
    MenuItem.Checked = SilenceEnabled

    ' Child of [Player] in the options:
    SDB.UI.AddOptionSheet "Silence between songs", Script.ScriptPath, "InitSheet", "SaveSheet", -2
End Sub

'---------------------
Sub ToggleSilence(p)
    SilenceEnabled = Not SilenceEnabled
    MenuItem.Checked = SilenceEnabled
    SDB.IniFile.BoolValue(AppTitle,"Enabled") = SilenceEnabled

    If SilenceEnabled then
        SDB.IniFile.BoolValue(AppTitle,"CrossfadeState") = SDB.Player.IsCrossfade
        SDB.Player.IsCrossfade = False
    Else
        SDB.Player.IsCrossfade = SDB.IniFile.BoolValue(AppTitle,"CrossfadeState")
    End If
    SilenceBetweenSongs
End Sub

'---------------------
Sub SilenceBetweenSongs
    If SilenceEnabled Then
       Script.RegisterEvent SDB, "OnPlay", "PlayerOnPlay"
       Script.RegisterEvent SDB, "OnTrackEnd", "PlayerTrackEnd"
    Else
        Script.UnregisterEvents SDB
        SDB.Player.StopAfterCurrent = False
    End If
End Sub

'---------------------
Sub InitTimer()
  Set GapTimer = SDB.CreateTimer(1000)
  GapTimer.Enabled = False
  Script.RegisterEvent GapTimer, "OnTimer", "GapOnTimer"
End Sub

'---------------------
Sub PlayerOnPlay()
   If SilenceEnabled Then
      If ExcludeGapless Then
         isGapless = CBool(InStr(Eval("SDB.Player.CurrentSong.Custom"&CustomFieldToUse), CustomFieldText))
         'SDB.MessageBox "Gapless = " & isGapless, mtInformation, Array(mbOK)
         SDB.Player.StopAfterCurrent = NOT isGapless
      Else
         SDB.Player.StopAfterCurrent = True
      End If
      CurrentTrack = SDB.Player.CurrentSongIndex   'Save current playing track position
   End If
   If  GapTimer.Enabled Then
      GapTimer.Enabled = False
      Set GapProgress = Nothing
   End If
End Sub

'---------------------
Sub PlayerTrackEnd()
   If SilenceEnabled And (SDB.Player.CurrentSongIndex+1 < SDB.Player.PlayListCount or SDB.Player.IsRepeat or SDB.Player.IsShuffle) Then
      If ExcludeGapless And isGapless Then
         GapTimer.Enabled = False
         Set GapProgress = Nothing
      Else
         Set GapProgress = SDB.Progress
         GapProgress.MaxValue = SilenceTime
         GapProgress.Text="Gap " & SilenceTime & " seconds."
         GapTimer.Enabled = True
      End If
   Else
      GapTimer.Enabled = False
      Set GapProgress = Nothing
   End If
End Sub

'---------------------
Sub GapOnTimer(Timer)
   If GapProgress.Value = 0 Then
      SDB.Player.Next
   End If
   If GapTimer.Enabled = True Then
      GapProgress.Increase
      GapProgress.Text="Gap " & SilenceTime - GapProgress.Value & " seconds."
   End If
   If GapProgress.Value >= GapProgress.MaxValue Then
      'If Repeat is ON, continue playing:
      If SDB.Player.IsRepeat Then
         SDB.Player.Play
      Else
         'Continue playing if Shuffle has not played all files:
         If CurrentTrack <> SDB.Player.CurrentSongIndex Then
            SDB.Player.Play
         End If
      End If
      GapTimer.Enabled = False
      Set GapProgress = Nothing
   End If
End Sub

'---------------------
Sub InitSheet(Sheet)
   Dim oPanel1, oCheck1, oSpin1, oPanel2, oCheck2, oField2, oString2

   With SDB.UI.NewLabel(Sheet)
      .Common.Left = 460
      .Common.Top = 5
      .Caption = "v" & Version
   End With

   With SDB.UI.NewLabel(Sheet)
      .Alignment = 2    'Center
      .Common.SetRect 85,30,280,40
      .Caption = "Adds silence between playing songs." & vbcrlf & _
                 "Can also be enabled/disabled through Play menu."
   End With

   Set oPanel1 = SDB.UI.NewGroupBox(Sheet)
   oPanel1.Common.SetRect 85,80,280,100
   oPanel1.Caption = "Delay between songs"

   Set oCheck1 = SDB.UI.NewCheckBox(oPanel1)
   With oCheck1
      .Caption = "Enable"
      .Common.Left = 25
      .Common.Top = 25
      .Common.ControlName = "ChEnable"
      .Checked = SilenceEnabled
   End With


   Set oSpin1 = SDB.UI.NewSpinEdit(oPanel1)
   With oSpin1
      .Common.Left = 25
      .Common.Top = 55
      .Common.Width = 45
      .MinValue = 1
      .MaxValue = 15
      .Common.ControlName = "EdLength"
      .Value = SilenceTime
   End With

   With SDB.UI.NewLabel(oPanel1)
      .caption = "second(s)"
      .Common.Left = 80
      .Common.Top = 58
   End With
   
   ' ---------------------------------------
   Set oPanel2 = SDB.UI.NewGroupBox(Sheet)
   oPanel2.Common.SetRect 85,200,280,123
   oPanel2.Caption = "Options"

   Set oCheck2 = SDB.UI.NewCheckBox(oPanel2)
   With oCheck2
      .Caption = "Exclude gapless tracks"
      .Common.Left = 25
      .Common.Top = 25
      .Common.Width = 220
      .Common.ControlName = "ChEnable2"
      .Checked = ExcludeGapless
   End With
   
   With SDB.UI.NewLabel(oPanel2)
      .caption = "Noted anywhere in:"
      .Common.Left = 25
      .Common.Top = 58
   End With
   
   Set oField2 = SDB.UI.NewDropDown(oPanel2)
   With oField2
      .Common.ControlName = "oFieldToUse"
      .Style = 2        'csDropDownList
      .Common.SetRect 150,55,100,21
      .AddItem "Custom1"
      .AddItem "Custom2"
      .AddItem "Custom3"
      .AddItem "Custom4"
      .AddItem "Custom5"
      .ItemIndex = CustomFieldToUse - 1
    End With  

   With SDB.UI.NewLabel(oPanel2)
      .caption = "As (text):"
      .Common.Left = 25
      .Common.Top = 83
   End With

   Set oString2 = SDB.UI.NewEdit(oPanel2)
   With oString2
      .Common.ControlName = "oFieldText"
      .Common.SetRect 120,80,130,21
      .Text = CustomFieldText
   End With
End Sub



'---------------------
Sub SaveSheet(Sheet)
    Dim v
    
    Set v = Sheet.Common.ChildControl("ChEnable")
    If v.Checked <> SilenceEnabled then
        ToggleSilence 0
    End If
    
    'Extact values:'
    SilenceTime = Sheet.Common.ChildControl("EdLength").Value
    ExcludeGapless = Sheet.Common.ChildControl("ChEnable2").Checked
    CustomFieldToUse = Sheet.Common.ChildControl("oFieldToUse").ItemIndex + 1
    CustomFieldText = Sheet.Common.ChildControl("oFieldText").Text
    If CustomFieldText="" Then CustomFieldText = DEFAULTCustomFieldText
    
    'Write values:
    'SDB.IniFile.BoolValue(AppTitle,"Enabled") = SilenceEnabled         'Not necessary
    SDB.IniFile.StringValue(AppTitle,"SilenceTime") = SilenceTime
    SDB.IniFile.BoolValue(AppTitle,"ExcludeGapless") = ExcludeGapless
    SDB.IniFile.StringValue(AppTitle,"CustomFieldToUse") = CustomFieldToUse
    SDB.IniFile.StringValue(AppTitle,"CustomFieldText") = CustomFieldText  
End Sub
